% SIFTSdemo3
%
% DESCRIPTION:
%  Example of matching between Local Spherical Descriptors of a parabolic
%  omnidirectional image and a corrupted and rotated version of the same
%  image
%
% VERSION:
%  1.1
%
% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%	  Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

global SIFTSLocalPath;
% Parabolic image:
imageStr = [SIFTSLocalPath '/TestImages/Parabolic_Image.jpg'];

% Radius of the disk image (depends on the sensor calibration)
R = 3.2;

% Spherical image size
sphSize = 1024;
 
% Compute spherical image
parabolicImage = imread(imageStr);
      origImg1 = KAIDANToSphere_colour(parabolicImage, R, sphSize);
       sphImg1 = double(rgb2gray(origImg1));
       sphImg1 = sphImg1 - min(min(sphImg1));
       sphImg1 = sphImg1/max(max(sphImg1));

% Compute rotated and corrupted spherical image
        angle = random('unif', 0, 2*pi);
     noiseStd = 0.05;
     origImg2 = rotateSphericalImage(origImg1, angle);
additiveNoise = random('norm',0,noiseStd*ones(sphSize));
      sphImg2 = double(rgb2gray(origImg2)) + additiveNoise;
      sphImg2 = sphImg2 - min(min(sphImg2));
      sphImg2 = sphImg2/max(max(sphImg2));

% Double-size image
% octaveOffset = -1;
% Original-size image
octaveOffset = 0;

% Number of intervals per octave
S = 3;

% Nominal and Base sigmas (in relative distance between pixels)
sigmaN = 0.5;
sigma0 = 3.0;

% Number of stages in the pyramid
numOctaves = floor(log2(sphSize))-5;

% Spherical Scale-Space and DoG computation
 SSS1 = sphericalScaleSpace(sphImg1, S, numOctaves, octaveOffset, sigma0, sigmaN);
SDoG1 = differenceOfGaussians(SSS1);
 SSS2 = sphericalScaleSpace(sphImg2, S, numOctaves, octaveOffset, sigma0, sigmaN);
SDoG2 = differenceOfGaussians(SSS2);

% Find local maxima/minima in the scale-space representation
[extremePoints1, numPoints1] = findLocalExtrema(SDoG1);
[extremePoints2, numPoints2] = findLocalExtrema(SDoG2);

% Compute Local Spherical Descriptors (LSD)
 numBinsPrincipalAngle = 36;
               numBins = 8;
   numSpatialDivisions = 4;
[LSD1, extremePoints1] = computeLSD(extremePoints1, SSS1, numBinsPrincipalAngle, numBins, numSpatialDivisions);
[LSD2, extremePoints2] = computeLSD(extremePoints2, SSS2, numBinsPrincipalAngle, numBins, numSpatialDivisions);

% Compute correspondences
      distRatio = 0.7;
integerMatching = true;
     samePoints = computeMatching(LSD1, LSD2, distRatio, integerMatching);

% Test repeatability and correct matching 
% (only if the transformation from one image to the other is a known
% rotation!)
[numTotal1, numTotal2, numRepeat] = testRotationRepeatability(extremePoints1, extremePoints2, sphSize, sphSize, angle);
[numTotal1, numTotal2, numSame, numCorrectMatch] = testRotationMatching(LSD1, LSD2, samePoints, sphSize, sphSize, angle);
 
% Show matchings
showMatchings(samePoints, extremePoints1, extremePoints2, origImg1, origImg2);

% Show matchings 1 by 1
% showMatchings1by1(samePoints, extremePoints1, extremePoints2, origImg1, origImg2);

% Show correct matchings in green and incorrect matchings in red
% (only if the transformation from one image to the other is a known
% rotation!)
showCorrectIncorrectMatchings(samePoints, extremePoints1, extremePoints2, origImg1, origImg2, angle);
