function [samePoint] = computeMatching(LSD1, LSD2, distRatio, integerMatching)
% [samePoint] = computeMatching(LSD1, LSD2, distRatio, integerMatching)
%
% DESCRIPTION:
%  Computes matching between two sets of Local Spherical Descriptors 
%
% VERSION:
%  1.1
%
% INPUT:
%  LSD1             = set 1 of Local Spherical Descriptors
%  LSD2             = set 2 of Local Spherical Descriptors
%  distRatio        = threshold for the ratio between the distances of the
%                     nearest neighbour and the second nearest neighbour
%  integerMatching  = if set to true, descriptors are converted to integers
%                     by doing "uint16(512.0*LSD)" before performing the
%                     matching
%
% OUTPUT:
%  samePoint  = list of Local Spherical Descriptors matched


% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    tic
    samePoint =  struct('index1',[],'index2',[],'point1',[],'point2',[]);
    numPoints1 = size(LSD1,1);
    numPoints2 = size(LSD2,1);

    disp('[SIFTS] Computing matching points...');
  
    for i=1:numPoints1
        dist2 = Inf(numPoints2,1);
        for j=1:numPoints2
            if (integerMatching)
                dist2(j) = sum((uint16(512.0*LSD1(i).gradientHistograms) - uint16(512.0*LSD2(j).gradientHistograms)).^2);
            else
                dist2(j) = sum((LSD1(i).gradientHistograms - LSD2(j).gradientHistograms).^2);
            end
        end
        [minV, minI] = min(dist2);
        dist2(minI) = Inf(1);
        min2V = min(dist2);
        if((minV/min2V)<distRatio)
            samePoint.index1 = [samePoint.index1; i];
            samePoint.index2 = [samePoint.index2; minI];
            samePoint.point1 = [samePoint.point1; LSD1(i).imgPoints];
            samePoint.point2 = [samePoint.point2; LSD2(minI).imgPoints];
        end
    end

    disp(sprintf('[SIFTS] \t%d matching points found', length(samePoint.index1)));
    disp('[SIFTS] Done!');
    t=toc;
    disp(sprintf('[SIFTS] Computation time: %f seconds\n',t));

end
