function [LSDescriptor, extremePoints] = getLSD(extremePoints, img, sigma, numBinsPrincipalAngle, numBins, numSpatialDivisions)
% [LSDescriptor, extremePoints] = getLSD(extremePoints, img, sigma, numBinsPrincipalAngle, numBins, numSpatialDivisions)
%
% DESCRIPTION:
%  Computes Local Spherical Descriptors of each considered extreme for a 
%  given stage of the spherical difference of Gaussians.
%
% VERSION:
%  1.1
%
% INPUT:
%  extremePoints         =  list of extrema of the given stage of the 
%                           spherical difference of Gaussians
%  img                   =  image of the spherical scale-space
%                           representation of the image that corresponds to
%                           the given stage of the spherical difference of
%                           Gaussians
%  sigma                 =  sigma of the given stage
%  numBinsPrincipalAngle =  number of bins of the histogram of orientations
%                           for the computation of the principal
%                           orientation
%  numBins               =  number of bins of the descriptor in the 
%                           orientation dimension
%  numSpatialDivisions   =  number of divisions of each of the two spatial 
%                           dimensions for the computation of the
%                           descriptor
%
% OUTPUT:
%  LSDescriptor   =  list of Local Spherical Descriptors for the given
%                    stage
%  extremePoints  =  "extremePoints" (input) with their corresponding
%                    principal orientation
%
% SEE ALSO:    computeLSD

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    numPoints = length(extremePoints.octave);
    [imgR, imgC, n] = size(img);
    principalAngle = zeros(numPoints,numBinsPrincipalAngle);
    numHists = numSpatialDivisions*numSpatialDivisions;
    
    % Compute principal angle
    % =======================
    
    dD = derivSphere(img, 2*pi/imgC, pi/imgR);
    gradAngles = atan2(dD{1},dD{2});
    gradAngles = mod(gradAngles, 2*pi);
    gradNorms = sqrt(dD{1}.^2 + dD{2}.^2);

    nRAngle = numPoints;
    R = 1.5*sigma;
    phiAngle = 0:2*pi/imgC:2*pi*(1-1/imgC);
    thetaAngle = pi/(2*imgR):pi/imgR:pi*(1-1/(2*imgR));

    numSpCol = ceil(R*imgC/(2*pi));
    vecCol = (-numSpCol-0.5:numSpCol+0.5);
    colCenter = pi*((imgC+1)/imgC);
    col = colCenter+vecCol*2.0*pi/imgC;

    numSpRow = ceil(R*imgR/pi);
    vecRow = (-numSpRow-0.5:numSpRow+0.5);
    rowCenter = pi/2.0;
    row = rowCenter+vecRow*pi/imgR;
    
    for i=1:nRAngle
        % Rotation of rowCenter-thetaOrig wrt y
        [rowAngle, colAngle] = computeRotatedCoords(    row, col, pi/2, pi/2, ...
                                                        rowCenter-((2*extremePoints.octaveOptPoints(i,1)-1)*pi/(2.0*imgR)));
        % Rotation of phiOrig-colCenter wrt z
        [rowAngle, colAngle] = computeRotatedCoordsVector(  rowAngle(:), colAngle(:), 0, 0, ...
                                                            extremePoints.octaveOptPoints(i,2)*2.0*pi/imgC - colCenter);

        [colAngle,rowAngle] = meshgrid( min(floor(colAngle*imgC/(2*pi))):max(ceil(colAngle*imgC/(2*pi))), ...
                                        min(floor(rowAngle*imgR/pi)):max(ceil(rowAngle*imgR/pi)));

        rowAngle(rowAngle==0) = 1;
        rowAngle(rowAngle==imgR+1) = imgR;
        
        colAngle(colAngle<1) = colAngle(colAngle<1)+imgC;
        colAngle(colAngle>imgC) = colAngle(colAngle>imgC)-imgC;

        % Calculate which histogram must be used
        for j=1:length(rowAngle(:))
            histBin = ceil(numBinsPrincipalAngle*gradAngles(rowAngle(j), colAngle(j))/(2*pi));
            if(histBin==0)
                histBin = 1;
            end
            % Add values to the histogram
            principalAngle(i,histBin) = principalAngle(i,histBin) + ...
                gradNorms(rowAngle(j),colAngle(j)) * gradientWeight(  sphericalDistance( ...
                                                                            extremePoints.octaveOptPoints(i,2)*2.0*pi/imgC, ...
                                                                            phiAngle(colAngle(j)), (1+2*extremePoints.octaveOptPoints(i,1))*pi/(2*imgR), ...
                                                                            thetaAngle(rowAngle(j))), ...
                                                                      extremePoints.sigma(i),1.5 );
        end
    end
    
    % Locate maximums of the histogram
    % Maximums are calculated according to fitted parabolas around histogram maximums
    % -1 => p1 ; 0 => p2 ; +1 => p3
    % y  = ax^2+bx+c
    % 2a = p1-2p2+p3
    % b  = (p3-p1)/2
    % c = p2
    % Maximum at x = -b/2a
    [maxV,ind] = max(principalAngle,[],2);
    startIndex = 1;
    pointInd = (1:length(ind))';

    tmpprincipalAngle = principalAngle;

    while(~isempty(ind))
    
        y1 = ind-1;
        y1(y1==0) = numBinsPrincipalAngle;
        y3 = ind+1;
        y3(y3==numBinsPrincipalAngle+1) = 1;

        numIndex = length(ind);
        
        for i=startIndex:startIndex+numIndex-1
            LSDescriptor(i) = struct('octave',extremePoints.octave(pointInd(i)),'stage',extremePoints.stage(pointInd(i)), ...
                'sigma',extremePoints.sigma(pointInd(i)),'octavePoints',(extremePoints.octavePoints(pointInd(i),:)),          ...
                'octaveOptPoints', (extremePoints.octaveOptPoints(pointInd(i),:)), 'imgPoints',(extremePoints.imgPoints(pointInd(i),:)), ...
                'imgOptPoints', (extremePoints.imgOptPoints(pointInd(i),:)), 'gradientAngle',0.0,'numHistograms',numHists,            ...
                'numBinsPerHistogram',numBins,'gradientHistograms',zeros(numHists*numBins,1),'numPoints',zeros(numHists,1));    

            i2 = i-startIndex+1;
            b = (principalAngle(pointInd(i),y3(i2)) - principalAngle(pointInd(i),y1(i2)))/2;
            a2 = principalAngle(pointInd(i),y1(i2)) - 2*principalAngle(pointInd(i),ind(i2)) + principalAngle(pointInd(i),y3(i2));

            extremePoints.gradientAngle(i,1) = mod((ind(i2)-b/a2)*2*pi/numBinsPrincipalAngle, 2*pi);
            if(startIndex ~= 1)
                extremePoints.octave(i,1) = extremePoints.octave(pointInd(i));
                extremePoints.stage(i,1) = extremePoints.stage(pointInd(i));
                extremePoints.sigma(i,1) = extremePoints.sigma(pointInd(i));
                extremePoints.octavePoints(i,:) = extremePoints.octavePoints(pointInd(i),:);
                extremePoints.octaveOptPoints(i,:) = extremePoints.octaveOptPoints(pointInd(i),:);
                extremePoints.imgPoints(i,:) = extremePoints.imgPoints(pointInd(i),:);
                extremePoints.imgOptPoints(i,:) = extremePoints.imgOptPoints(pointInd(i),:);
            end
            
            LSDescriptor(i).gradientAngle = extremePoints.gradientAngle(i,1);

            % Detected maximum and neighbours cannot be detected again as maximums
            tmpprincipalAngle(pointInd(i),ind(i2)) = 0.0;
            tmpprincipalAngle(pointInd(i),y1(i2)) = 0.0;
            tmpprincipalAngle(pointInd(i),y3(i2)) = 0.0;
          
        end
        
        startIndex = startIndex+numIndex;
        
        [tmpMaxV,tmpInd]= max(tmpprincipalAngle,[],2);
        ind = tmpInd(tmpMaxV>0.8*maxV);
        pointInd = [pointInd; find(tmpMaxV>0.8*maxV)];
        
    end

    % Compute SSIFT descriptors (ROTATING 4 CORNERS FROM EQUATOR)
    % ===========================================================
    
    numPoints = length(extremePoints.octave);
    
    for i=1:numPoints
        %Select rotated region
        rowCenter = LSDescriptor(i).octaveOptPoints(1);
        rowLimit = max(round(3.0*sigma*imgR/pi)+0.5, 15.5);
        row = rowCenter+(-rowLimit:rowLimit);

        colCenter = LSDescriptor(i).octaveOptPoints(2);

        colLimit1 = colCenter+7.5;
        while((colLimit1-colCenter < imgC/2) && (sphericalDistance(colCenter*2.0*pi/imgC, colLimit1*2.0*pi/imgC, (2*row(1)-1)*pi/(2.0*imgR), (2*row(1)-1)*pi/(2.0*imgR))<=3.0*sigma))
            colLimit1 = colLimit1+1;
        end
        colLimit2 = colCenter+7.5;
        while((colLimit2-colCenter < imgC/2) && (sphericalDistance(colCenter*2.0*pi/imgC, colLimit2*2.0*pi/imgC, (2*row(end)-1)*pi/(2.0*imgR), (2*row(end)-1)*pi/(2.0*imgR))<=3.0*sigma))
            colLimit2 = colLimit2+1;
        end
        
        colLimit = max(colLimit1, colLimit2)-colCenter;
        col = colCenter+(-colLimit:colLimit);
        col(col<0) = col(col<0)+imgC;
        col(col>imgC) = col(col>imgC)-imgC;
        
        [newTheta, newPhi] = computeRotatedCoords((2*row-1)*pi/(2.0*imgR), (col-1)*2.0*pi/imgC, (2*rowCenter-1)*pi/(2.0*imgR), ...
                                              (colCenter-1)*2.0*pi/imgC, LSDescriptor(i).gradientAngle);

        newRow = round(newTheta*imgR/pi-0.5)+1;
        newRow(newRow==0) = 1;
        newRow(newRow==imgR+1) = imgR;
        newCol = round(newPhi*imgC/(2*pi))+1;
        newCol(newCol<1) = newCol(newCol<1)+imgC;
        newCol(newCol>imgC) = newCol(newCol>imgC)-imgC;

        regionAngs = mod(gradAngles(newRow + (newCol-1)*imgR)-LSDescriptor(i).gradientAngle,2*pi);
        regionNorms = gradNorms(newRow + (newCol-1)*imgR);

        %Compute descriptor
        LSDescriptor(i).gradientHistograms = computeRotatedDescriptor(regionAngs, regionNorms, numSpatialDivisions, numBins);

        % Normalize + Threshold 0.2 + Normalize
        histNorm = sqrt(sum(LSDescriptor(i).gradientHistograms(:).^2))+eps;
        LSDescriptor(i).gradientHistograms(:) = LSDescriptor(i).gradientHistograms(:)/histNorm;

        LSDescriptor(i).gradientHistograms(LSDescriptor(i).gradientHistograms>0.2) = 0.2;

        histNorm = sqrt(sum(LSDescriptor(i).gradientHistograms(:).^2))+eps;
        LSDescriptor(i).gradientHistograms(:) = LSDescriptor(i).gradientHistograms(:)/histNorm;
    end

end
