function [img] = drawMarks(img, extrema)
% [img] = drawMarks(img, extrema)
%
% DESCRIPTION:
%  Draws extrema of the spherical difference of Gaussian. For each extreme,
%  a circle of radius 1.5*sigma and its principal orientation is plotted
%
% VERSION:
%  1.1
%
% INPUT:
%  img      =  omnidirectional image mapped on the sphere
%  extrema  =  list of extrema of the spherical difference of Gaussians
%
% OUTPUT:
%  img  =  omnidirectional image mapped on the sphere with the plots of the
%          extrema

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    if (isempty(extrema.octave))
        return
    end

    [imgR, imgC, n] = size(img);

    R = 1.5*extrema.sigma;

    [phi,theta,phiValues,thetaValues] = computeRegions(extrema,imgR,imgC,0,R);
    [x,y] = computeRegionIndex(phiValues, thetaValues, phi, theta, imgC);
    numPoints = size(extrema.octave,1);

    % Draw circle
    for j=1:numPoints
        color = ceil(random('unif',0,5));
        for i=1:50
            x1 = x(j,i);
            y1 = y(j,i);
            x2 = x(j,i+1);
            y2 = y(j,i+1);
            img = drawPoint(x1, y1, img, imgC, imgR, color);
            if(x1>=x2 && y1>=y2)
                sector = 1;
            elseif(x1>=x2 && y1<y2)
                sector = 2;
            elseif(x1<x2 && y1>=y2)
                sector = 3;
            else
                sector = 4;
            end

            while(x1-x2~=0 || y1-y2~=0)

                if(sector == 1)
                    P = [x1-1, y1; x1-1, y1-1; x1, y1-1];
                elseif(sector == 2)
                    P = [x1-1, y1; x1-1, y1+1; x1, y1+1];
                elseif(sector == 3)
                    P = [x1, y1-1; x1+1, y1-1; x1+1, y1];
                else
                    P = [x1+1, y1; x1+1, y1+1; x1, y1+1];
                end
                [val,ind] = min(sqrt((P(:,1)-x2).^2 + (P(:,2)-y2).^2) + sqrt((P(:,1)-x(j,i)).^2 + (P(:,2)-y(j,i)).^2));
                x1 = P(ind,1);
                y1 = P(ind,2);
                img = drawPoint(x1, y1, img, imgC, imgR, color);          
            end
        end
    end
    
    % Draw orientation
    t = 0:pi/(2*imgC):pi*(0.5-0.5/imgC);
    
    thetaOrig = theta(extrema.imgPoints(:,1));
    phiOrig = phi((extrema.imgPoints(:,2)));
    Alpha = extrema.gradientAngle(:)';
    
    % Each row corresponds to a point
    dPhi = asin((sin(t)')*(sin(Alpha)./sin(thetaOrig)));
    phiP = repmat(phiOrig,[imgC,1]) + dPhi;
    phiP = phiP';
    thetaP(1,:) = thetaOrig;
    thetaP(2:imgC,:) = acos((cos(t(2:imgC))')*cos(thetaOrig)-(sin(t(2:imgC))')*(sin(thetaOrig).*cos(Alpha)));
    thetaP = thetaP';
  
    [x,y] = computeRegionIndex(phiP, thetaP, phi, theta, imgC);

    color = -1;    
    for j=1:numPoints
        border=0;
        for i=1:imgC-1
            if(border==1)
                break;
            end
            x1 = x(j,i);
            y1 = y(j,i);
            x2 = x(j,i+1);
            y2 = y(j,i+1);
            img = drawPoint(x1, y1, img, imgC, imgR, color);
            if(x1>=x2 && y1>=y2)
                sector = 1;
            elseif(x1>=x2 && y1<y2)
                sector = 2;
            elseif(x1<x2 && y1>=y2)
                sector = 3;
            else
                sector = 4;
            end

            while((x1-x2~=0 || y1-y2~=0) && border==0)

                if(sector == 1)
                    P = [x1-1, y1; x1-1, y1-1; x1, y1-1];
                elseif(sector == 2)
                    P = [x1-1, y1; x1-1, y1+1; x1, y1+1];
                elseif(sector == 3)
                    P = [x1, y1-1; x1+1, y1-1; x1+1, y1];
                else
                    P = [x1+1, y1; x1+1, y1+1; x1, y1+1];
                end
                [val,ind] = min(sqrt((P(:,1)-x2).^2 + (P(:,2)-y2).^2) + sqrt((P(:,1)-x(j,i)).^2 + (P(:,2)-y(j,i)).^2));
                x1 = P(ind,1);
                y1 = P(ind,2);
                
                if(x1<1)
                    x1Test = imgC+x1;
                elseif(x1>imgC)
                    x1Test = x1-imgC;
                else
                    x1Test = x1;
                end
                if(y1<1)
                    y1Test = abs(y1);
                elseif(y1>imgR)
                    y1Test = 2*imgR-y1;
                else
                    y1Test = y1;
                end

                if(sphericalDistance(phiOrig(j), phi(x1Test), thetaOrig(j), theta(y1Test))>R(j))
                    border=1;
                else
                    img = drawPoint(x1, y1, img, imgC, imgR, color);
                end
            end
        end
    end

    %Mark Detected Point
    for j=1:numPoints
        img = drawPoint(extrema.imgPoints(j,2), extrema.imgPoints(j,1), img, imgC, imgR, 0);
        img = drawPoint(extrema.imgPoints(j,2), extrema.imgPoints(j,1)+1, img, imgC, imgR, 0);
        img = drawPoint(extrema.imgPoints(j,2), extrema.imgPoints(j,1)-1, img, imgC, imgR, 0);
        img = drawPoint(extrema.imgPoints(j,2)+1, extrema.imgPoints(j,1), img, imgC, imgR, 0);
        img = drawPoint(extrema.imgPoints(j,2)-1, extrema.imgPoints(j,1), img, imgC, imgR, 0);
    end
    
end
