function [out] = sphericalGaussianFilter(in, sigmaS2)
% [out] = sphericalGaussianFilter(in, sigmaS2)
%
% DESCRIPTION:
%  Computes the convolution of an image with a spherical Gaussian filter.
%
% VERSION:
%  1.1
%
% INPUT:
%  in       =  omnidirectional input image mapped on the sphere
%  sigmaS2  =  sigma of the spherical Gaussian filter
%
% OUTPUT:
%  out  =  output image (convolution of the input image with a spherical
%          Gaussian filter with sigma=sigmaS2)

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    % spherical Fourier transform of data
    Fin = fst(double(in));
    L   = size(Fin,1);
    L2  = 2*L;
    l   = 0:L-1;

    % create and apply spherical Gaussian filter
    G = exp(-0.5*l.*(l+1)*sigmaS2^2);
    mask = ilmshape(repmat(G(1,:),L2-1,1));
    out = real(ifst(Fin .* mask));
end
