function [H] = computeTransform(sphImg, planarImg, sphPoints, planarPoints)
% [H] = computeTransform(sphImg, planarImg, sphPoints, planarPoints)
%
% DESCRIPTION:
%  Computes, given a set of matchings, the transformation that sends points
%  in a planar image to points in a spherical one. After computing the
%  transformation, the user is asked to mark the perimeter of the object
%  in the planar image (by clicking on its vertexes and pressing RETURN
%  when finished) in order to show the transformed perimeter in the
%  spherical image.
%  See Section 4 of the REFERENCE for further details.
%
% VERSION:
%  1.1
%
% INPUT:
%  sphImg        =  omnidirectional image mapped on the sphere
%  planarImg     =  planar image
%  sphPoints     =  points on the spherical image matched with points on
%                   the planar one
%  planarPoints  =  points on the planar image matched with points on the
%                   spherical one
%
% OUTPUT:
%  H  =  computed transformation
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.

    disp('[SIFTS] Computing transformation...');

    [sphImgH, sphImgW, n] = size(sphImg);
    [planarImgH, planarImgW, n] = size(planarImg);

    % set RANSAC options
    clear options;
    options.epsilon = 0.4;
    options.P_inlier = 0.7;
    options.sigma = 0.01;
    options.validateTheta_fun = @validateTransform;
    options.est_fun = @estimateTransform;
    options.man_fun = @errorTransform;
    options.mode = 'MSAC';
    options.min_iters = 1000;
    options.fix_seed = false;
    options.reestimate = true;
    options.stabilize = true;

    planarCenterX = planarPoints(:,2)-planarImgW/2;
    planarCenterY = planarPoints(:,1)-planarImgH/2;
    scale = max(planarImgW,planarImgH);
    planarImgX = planarCenterX/scale;
    planarImgY = planarCenterY/scale;

    pointCorrespondence = zeros(6,length(sphPoints(:,2)));
    for i=1:length(sphPoints(:,2))
        thetai = (2*sphPoints(i,1)+1)*pi/(2*sphImgH);
        phii = sphPoints(i,2)*2*pi/sphImgW;
        pointCorrespondence(:,i) = [planarImgX(i),planarImgY(i),1,sin(thetai)*cos(phii),sin(thetai)*sin(phii),cos(thetai)]';
    end

    % run RANSAC
    [results, options] = RANSAC(pointCorrespondence, options);

    H = [results.Theta(1:3)';results.Theta(4:6)';results.Theta(7:9)'];

    index = find(results.CS);
    for i=1:length(index)
        planarImg(floor(planarPoints(index(i),1))-1:ceil(planarPoints(index(i),1))+1,floor(planarPoints(index(i),2))-1:ceil(planarPoints(index(i),2))+1,1)=0;
        planarImg(floor(planarPoints(index(i),1))-1:ceil(planarPoints(index(i),1))+1,floor(planarPoints(index(i),2))-1:ceil(planarPoints(index(i),2))+1,2)=255;
        planarImg(floor(planarPoints(index(i),1))-1:ceil(planarPoints(index(i),1))+1,floor(planarPoints(index(i),2))-1:ceil(planarPoints(index(i),2))+1,3)=0;
    end

    msg = sprintf('\n[SIFTS]\t Segment a region of the planar image by clicking on its vertexes');
    disp(msg);
    msg = sprintf('[SIFTS]\t Press RETURN after clicking on the last vertex\n');
    disp(msg);
    
    figure;
    imshow(planarImg);
    [xInput,yInput] = ginput;
    for i=1:4
        if (i==4)
            iplus = 1;
        else
            iplus = i+1;
        end
        h = imline(gca, [xInput(i) xInput(iplus)], [yInput(i) yInput(iplus)]);
        setColor(h,[0 1 0]);
    end
    
    square = {};
    xInputCenter = xInput-planarImgW/2;
    yInputCenter = yInput-planarImgH/2;
    planarInputImgX = xInputCenter/scale;
    planarInputImgY = yInputCenter/scale;
    
    numDivisions = 10;
    
    for i=1:length(planarInputImgX)
        if (i==length(planarInputImgX))
            iniX = planarInputImgX(i);
            iniY = planarInputImgY(i);
            finX = planarInputImgX(1);
            finY = planarInputImgY(1);
        else
            iniX = planarInputImgX(i);
            iniY = planarInputImgY(i);
            finX = planarInputImgX(i+1);
            finY = planarInputImgY(i+1);
        end
        w = finX-iniX;
        h = finY-iniY;
        for j=0:numDivisions
            square{(i-1)*numDivisions+j+1} = [planarInputImgX(i)+j*w/numDivisions,planarInputImgY(i)+j*h/numDivisions,1];
        end
    end

    mask = zeros(sphImgH,sphImgW,n);
    maskPlanar = zeros(planarImgH,planarImgW,n);
    for i=1:length(square)
        squareTransf = H*square{i}';
        squareTransfNorm = squareTransf/norm(squareTransf);
        [phi,theta,rho] = cart2sph(squareTransfNorm(1),squareTransfNorm(2),squareTransfNorm(3));
        theta = pi/2 - theta;
        phi = mod(phi, 2*pi);
        row = round(theta*sphImgH/pi - 0.5);
        col = round(phi*sphImgW/(2*pi));

        xOrig = round(square{i}(1)*scale+planarImgW/2);
        yOrig = round(square{i}(2)*scale+planarImgH/2);

        if (i>1)
            samePoints.index1 = 0;
            samePoints.index2 = 0;
            samePoints.point1 = [rowOld colOld];
            samePoints.point2 = [row col];
            mask = plotLine(mask, samePoints);
            samePoints.point1 = [rowOld-1 colOld-1];
            samePoints.point2 = [row-1 col-1];
            mask = plotLine(mask, samePoints);
            samePoints.point1 = [rowOld+1 colOld+1];
            samePoints.point2 = [row+1 col+1];
            mask = plotLine(mask, samePoints);

            samePoints.point1 = [rowPlanarOld colPlanarOld];
            samePoints.point2 = [yOrig xOrig];
            maskPlanar = plotLine(maskPlanar, samePoints);
            samePoints.point1 = [rowPlanarOld-1 colPlanarOld-1];
            samePoints.point2 = [yOrig-1 xOrig-1];
            maskPlanar = plotLine(maskPlanar, samePoints);
            samePoints.point1 = [rowPlanarOld+1 colPlanarOld+1];
            samePoints.point2 = [yOrig+1 xOrig+1];
            maskPlanar = plotLine(maskPlanar, samePoints);
        end

        rowOld = row;
        colOld = col;
        rowPlanarOld = yOrig;
        colPlanarOld = xOrig;
    end

    index = find(mask(:,:,1));
    sphImg(index)=0;
    sphImg(index+sphImgH*sphImgW)=128;
    sphImg(index+2*sphImgH*sphImgW)=255;

    index = find(maskPlanar(:,:,1));
    planarImg(index)=0;
    planarImg(index+planarImgH*planarImgW)=128;
    planarImg(index+2*planarImgH*planarImgW)=255;

    figure;imshow(planarImg);
    figure;yashow(sphImg, 'spheric');
    figure;imshow(sphImg);
    
    msg = sprintf('[SIFTS] Done!\n');
    disp(msg);

end
