function [S1] = KAIDANToSphere_colour(I, R, ImSize)
% [S1] = KAIDANToSphere_colour(I, R, ImSize)
%
% DESCRIPTION:
%  Maps an omnidirectional parabolic image on the sphere
%
% VERSION:
%  1.1
%
% INPUT:
%  I       =  omnidirectional parabolic image
%  R       =  outer radius of the disc
%  ImSize  =  desired size of the spherical grid. Must be a power of 2, for
%             example 1024
%
% OUTPUT:
%  S1  =  omnidirectional image mapped on the sphere

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    tic
    disp('[SIFTS] Mapping parabolic image on the sphere...');

    [Iy, Ix] = size(I(:,:,1));
    mask = ones(Ix,Iy);
    [Xd,Yd] = meshgrid(-R:2.0*R/double(Ix-1):R,-R:2.0*R/double(Iy-1):R);
    mask(Xd.^2 + Yd.^2 >= R^2) = 0.0;
    mask = uint8(mask);
    [phi, theta] = sphgrid(ImSize,ImSize); % uses Yawtb 

    xij = 2*tan(theta/2).*cos(phi);
    yij = 2*tan(theta/2).*sin(phi);

    S_Ir = interp2(Xd,Yd,double(I(:,:,1).*mask),xij,yij,'*cubic'); % through cubic interpolation as defined in MATLAB
    %S_Ir = interp2(Xd,Yd,double(data_r),xij,yij,'*linear'); % through linear interpolation as defined in MATLAB
    %S_Ir = interp2(Xd,Yd,double(data_r),xij,yij,'*nearest'); % through nearest neighbour interpolation as defined in MATLAB
    S_Ir(isnan(S_Ir)) = 0;

    S_Ig = interp2(Xd,Yd,double(I(:,:,2).*mask),xij,yij,'*cubic'); % through cubic interpolation as defined in MATLAB
    %S_Ig = interp2(Xd,Yd,double(data_g),xij,yij,'*linear'); % through linear interpolation as defined in MATLAB
    %S_Ig = interp2(Xd,Yd,double(data_g),xij,yij,'*nearest'); % through nearest neighbour interpolation as defined in MATLAB
    S_Ig(isnan(S_Ig)) = 0;

    S_Ib = interp2(Xd,Yd,double(I(:,:,3).*mask),xij,yij,'*cubic'); % through cubic interpolation as defined in MATLAB
    %S_Ib = interp2(Xd,Yd,double(data_b),xij,yij,'*linear'); % through linear interpolation as defined in MATLAB
    %S_Ib = interp2(Xd,Yd,double(data_b),xij,yij,'*nearest'); % through nearest neighbour interpolation as defined in MATLAB
    S_Ib(isnan(S_Ib)) = 0;

    S1(:,:,1) = uint8(S_Ir);
    S1(:,:,2) = uint8(S_Ig);
    S1(:,:,3) = uint8(S_Ib);

    disp('[SIFTS] Done!');
    t=toc;
    disp(sprintf('[SIFTS] Computation time: %f seconds\n',t));

end
