function dD = derivSphereScale(Img0, Img1, Img2, row, col, phiRes, thetaRes, sigmaRes)
% dD = derivSphereScale(Img0, Img1, Img2, row, col, phiRes, thetaRes, sigmaRes)
%
% DESCRIPTION:
%  Computes partial derivatives with respect to phi, theta and scale at a
%  given position
%
% VERSION:
%  1.1
%
% INPUT:
%  Img0      =  grayscale value of the omnidirectional image mapped on the 
%               sphere at the given position (lower scale)
%  Img1      =  grayscale value of the omnidirectional image mapped on the 
%               sphere at the given position 
%  Img2      =  grayscale value of the omnidirectional image mapped on the 
%               sphere at the given position (upper scale)
%  row       =  row in the spherical grid
%  col       =  column in the spherical grid
%  phiRes    =  resolution in phi
%  thetaRes  =  resolution in theta
%  sigmaRes  =  resolution in sigma
%
% OUTPUT:
%  dD  =  derivatives with respect to phi (dD(1)), theta (dD(2)) and scale
%         (dD(3))

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    [Iy, Ix] = size(Img1);

    %Partial derivative with respect to Phi
    [cola] = returnGoodCol(col+1, Ix);
    [colb] = returnGoodCol(col-1, Ix);
    dD(1,1) = (1.0/(sin(row*thetaRes)))*((Img1(row, cola) - Img1(row, colb))/(2*phiRes));
    
    %Partial derivative with respect to Theta
    [rowa,cola] = returnGoodRowCol(row+1, col, Iy, Ix);
    [rowb,colb] = returnGoodRowCol(row-1, col, Iy, Ix);
    dD(2,1) = (Img1(rowa, cola) - Img1(rowb, colb))/(2*thetaRes);
    
    %Partial derivative with respect to Sigma
    dD(3,1) = (Img2(row, col) - Img0(row, col))/(2*sigmaRes);

end
