function [extrema, numExtrema] = findLocalExtrema(SDoG)
% [extrema, numExtrema] = findLocalExtrema(SDoG)
%
% DESCRIPTION:
%  Finds local extrema of the spherical difference of Gaussians
%
% VERSION:
%  1.1
%
% INPUT:
%  SDoG  =  spherical difference of Gaussians
%
% OUTPUT:
%  extrema     =  list of extrema of the spherical difference of 
%                 Gaussians 
%  numExtrema  =  number of extrema found

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    tic
    extrema = struct('octave',[],'stage',[],'sigma',[],'octavePoints',[],'octaveOptPoints',[],'imgPoints',[],'imgOptPoints',[],'gradientAngle',[]);

    disp('[SIFTS] Finding local extrema...');

    r = 10; % threshold for the ratio between principal curvatures
    
    for k=1:SDoG.numOctaves
        for l=2:SDoG.numStagesPerOctave-1
            
            Img0 = SDoG.octaves(k).octaveContents(l-1).image;
            Img1 = SDoG.octaves(k).octaveContents(l).image;
            Img2 = SDoG.octaves(k).octaveContents(l+1).image;
            contrastVal = 0.02/((SDoG.octaves(k).k^SDoG.octaves(k).octaveContents(l).stage)*(2^SDoG.octaves(k).octave));
            
            [Ny,Nx]=size(Img0);
            
            for i=2:Ny-1
                for j=1:Nx
                    isMax = -2;
                    isMin = -2;
                    if (j-1==0)
                        jmin = Nx;
                        jmax = j+1;
                    elseif (j==Nx)
                        jmax = 1;
                        jmin = j-1;
                    else
                        jmin = j-1;
                        jmax = j+1;
                    end

                    for m=-1:1
                        if((i+m==0) || (i+m==Ny+1))
                            % In this case, we consider that the line 1 or Ny is
                            % repeated at 0 or Ny+1 => it is the same than comparing
                            % for m=0 => it is not necessary to compare
                            % here because it would be done twice.
                            if(isMax == m-1)
                                isMax = isMax+1;
                            end
                            if(isMin == m-1)
                                isMin = isMin+1;
                            end
                        else
                            if(isMax == m-1)
                                if(Img1(i,j)>Img2(i+m,jmin) && Img1(i,j)>Img2(i+m,j) && Img1(i,j)>Img2(i+m,jmax))
                                    if(Img1(i,j)>Img0(i+m,jmin) && Img1(i,j)>Img0(i+m,j) && Img1(i,j)>Img0(i+m,jmax))
                                        if(Img1(i,j)>Img1(i+m,jmin) && Img1(i,j)>Img1(i+m,jmax) && (m==0 || Img1(i,j)>Img1(i+m,j)))
                                            isMax = isMax+1;
                                        end
                                    end
                                end
                            end
                            if(isMin == m-1)
                                if(Img1(i,j)<Img2(i+m,jmin) && Img1(i,j)<Img2(i+m,j) && Img1(i,j)<Img2(i+m,jmax))
                                    if(Img1(i,j)<Img0(i+m,jmin) && Img1(i,j)<Img0(i+m,j) && Img1(i,j)<Img0(i+m,jmax))
                                        if(Img1(i,j)<Img1(i+m,jmin) && Img1(i,j)<Img1(i+m,jmax) && (m==0 || Img1(i,j)<Img1(i+m,j)))
                                            isMin = isMin+1;
                                        end
                                    end
                                end
                            end
                        end
                        
                        if(isMax~=m && isMin~=m)
                            break;
                        end
                    end
                    if(isMax==1 || isMin==1)
                        % Check with interpolation
                        [check, dD, H, rowOpt, colOpt, rowFin, colFin] = interpolateExtreme(Img0, Img1, Img2, i, j, SDoG.octaves(k).octave, l-2, contrastVal);
                        if( check == true )
                            % Edge responses elimination
                            is_edge_like = checkEdge(r, H);
                            if( is_edge_like == 1)
                                %Add point to the list
                                extrema.octave(end+1,1) = SDoG.octaves(k).octave;
                                extrema.stage(end+1,1) = SDoG.octaves(k).octaveContents(l).stage;
                                extrema.sigma(end+1,1) = SDoG.octaves(k).octaveContents(l).sigma;
                                extrema.octavePoints(end+1,:) = [rowFin,colFin];
                                extrema.octaveOptPoints(end+1,:) = [rowOpt,colOpt];
                                extrema.imgPoints(end+1,:) = round([(2^(SDoG.octaves(k).octave))*rowFin,(2^(SDoG.octaves(k).octave))*colFin]);
                                extrema.imgOptPoints(end+1,:) = [(2^(SDoG.octaves(k).octave))*rowOpt,(2^(SDoG.octaves(k).octave))*colOpt];
                                extrema.gradientAngle(end+1,1) = 0;
                            end
                        end
                    end
                end
            end
        end
    end
    numExtrema = length(extrema.octave);
    disp(sprintf('[SIFTS] \t%d local extrema found!',numExtrema));
    disp('[SIFTS] Done!');
    t=toc;
    disp(sprintf('[SIFTS] Computation time: %f seconds\n',t));
end
