function H = hessianSphere(Img0, Img1, Img2, row, col, phiRes, thetaRes, sigmaRes)
% [newTheta, newPhi] = computeRotatedCoordsVector(theta, phi, thetaCenter, phiCenter, angle)
%
% DESCRIPTION:
%  Computes the Hessian matrix at a given position in the spherical grid
%  and scale.
%
% VERSION:
%  1.1
%
% INPUT:
%  Img0      =  grayscale omnidirectional image mapped on the sphere (lower
%               scale)
%  Img1      =  grayscale omnidirectional image mapped on the sphere
%  Img2      =  grayscale omnidirectional image mapped on the sphere (upper
%               scale)
%  row       =  row in the spherical grid
%  col       =  column in the spherical grid
%  phiRes    =  resolution in phi
%  thetaRes  =  resolution in theta
%  sigmaRes  =  resolution in sigma
%
% OUTPUT:
%  H  =  Hessian matrix

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    [Iy, Ix] = size(Img1);

    % Definitions
    H = zeros(3,3);
    pix_val = Img1(row, col);

    % Computation of derivatives
    [cola] = returnGoodCol(col+1, Ix);
    [colb] = returnGoodCol(col-1, Ix);
    dPhiPhi = (1.0/((sin(row*thetaRes))^2))*((Img1(row, cola) + Img1(row, colb) - 2*pix_val)/(phiRes^2));

    [rowa,cola] = returnGoodRowCol(row+1, col, Iy, Ix);
    [rowb,colb] = returnGoodRowCol(row-1, col, Iy, Ix);
    dThetaTheta = (Img1(rowa, cola) + Img1(rowb, colb) - 2*pix_val)/(thetaRes^2);

    dSigmaSigma = (Img2(row, col) + Img0(row, col) - 2*pix_val)/(sigmaRes^2);

    [rowaa,colaa] = returnGoodRowCol(row+1, col+1, Iy, Ix);
    [rowbb,colbb] = returnGoodRowCol(row-1, col-1, Iy, Ix);
    [rowab,colab] = returnGoodRowCol(row+1, col-1, Iy, Ix);
    [rowba,colba] = returnGoodRowCol(row-1, col+1, Iy, Ix);
    dPhiTheta = (1.0/(sin(row*thetaRes)))*((Img1(rowaa, colaa) - Img1(rowab, colab) - Img1(rowba, colba) + Img1(rowbb, colbb))/(4*phiRes*thetaRes));
    dPhiSigma = (1.0/(sin(row*thetaRes)))*((Img2(rowa, cola) - Img2(rowb, colb) - Img0(rowa, cola) + Img0(rowb, colb))/(4*phiRes*sigmaRes));
    dThetaSigma = (Img2(rowa, cola) - Img2(rowb, colb) - Img0(rowa, cola) + Img0(rowb, colb))/(4*thetaRes*sigmaRes);

    % Assignment to matrix
    H(1,1) = dPhiPhi;
    H(1,2) = dPhiTheta;
    H(1,3) = dPhiSigma;
    H(2,1) = dPhiTheta;
    H(2,2) = dThetaTheta;
    H(2,3) = dThetaSigma;
    H(3,1) = dPhiSigma;
    H(3,2) = dThetaSigma;
    H(3,3) = dSigmaSigma;

end
