function [check, dD, H, rowOpt, colOpt, rowEnd, colEnd] = interpolateExtreme(Img0, Img1, Img2, row, col, octave, intvl, contrastVal)
% [check, dD, H, rowOpt, colOpt, rowEnd, colEnd] = interpolateExtreme(Img0, Img1, Img2, row, col, octave, intvl, contrastVal)
%
% DESCRIPTION:
%  Performs scale space extrema interpolation on location and scale,
%  discarding points with low contrast
%
% VERSION:
%  1.1
%
% INPUT:
%  Img0         =  grayscale omnidirectional image mapped on the sphere (lower
%                  scale)
%  Img1         =  grayscale omnidirectional image mapped on the sphere
%  Img2         =  grayscale omnidirectional image mapped on the sphere (upper
%                  scale)
%  row          =  row in the spherical grid
%  col          =  column in the spherical grid
%  octave       =  octave number
%  intvl        =  scale in the octave
%  contrastVal  =  threshold for the contrast
%
% OUTPUT:
%  check   =  true if the extreme can be considered, false otherwise
%  dD      =  partial derivatives with respect to phi, theta and scale
%  H       =  Hessian matrix
%  rowOpt  =  row in the spherical grid after iterative process
%  colOpt  =  column in the spherical grid after iterative process
%  rowEnd  =  row in the spherical grid after iterative process (integer)
%  colEnd  =  col in the spherical grid after iterative process (integer)

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    check = false;
    [Iy, Ix] = size(Img1);

    vSpacing = 0.5*pi/Iy;
    hSpacing = pi/Ix;

    i=0;
    while(check==false)
        [xi, xr, xc, dD, H] = interpolateStep(Img0, Img1, Img2, row, col, octave, intvl);
        if ((norm(xr) > vSpacing || norm(xc) > hSpacing ) && i<5)%|| norm(xi) > 0.5 )
            if (norm(xc) > hSpacing)
                col = returnGoodCol(col+sign(xc),Ix);
            end
            if (norm(xr) > vSpacing)
                [row,col] = returnGoodRowCol(row+sign(xr), col, Iy, Ix);
            end
            i = i+1;
        else
            if(norm(xr) > vSpacing || norm(xc) > hSpacing )
                % Bad convergence => we use the original point
                c = col;
                r = row;
                z = intvl;
            else
                % Good convergence => we use the corrected point
                c = col + Ix*xc/(2*pi);
                r = row + Iy*xr/pi;
                z = intvl + xi;
                [r,c] = returnGoodRowCol(r+1, c, Iy+2, Ix);
                r = r-1;
            end
            check = true;
            i=0;
        end
    end

    contrast = interpolateContrast(Img1(row, col), xr, xc, xi, dD);
    if(abs(contrast) < contrastVal)
        % return false if contrast is too low
        check = false;
        dD = zeros(3,1);
        H = zeros(3,3);
        rowOpt = 0;
        colOpt = 0;
        rowEnd = 0;
        colEnd = 0;
    else
        rowOpt = r;
        colOpt = c;
        [rowEnd,colEnd] = returnGoodRowCol(round(r), round(c), Iy, Ix);
        check = true;
   end
end
