function [xi, xr, xc, dD, H] = interpolateStep(Img0, Img1, Img2, row, col, octave, intvl)
% [xi,xr,xc,dD,H] = interpolateStep(Img0,Img1,Img2,row,col,octave,intvl)
%
% DESCRIPTION:
%  Computes offsets to the extreme
%
% VERSION:
%  1.1
%
% INPUT:
%  Img0    =  grayscale omnidirectional image mapped on the sphere (lower
%             scale)
%  Img1    =  grayscale omnidirectional image mapped on the sphere
%  Img2    =  grayscale omnidirectional image mapped on the sphere (upper
%             scale)
%  row     =  row number of the spherical grid
%  col     =  column number of the spherical grid
%  octave  =  octave number (not used)
%  intvl   =  stage number of the given octave (not used)
%
% OUTPUT:
%  xi  =  offset on scale dimension
%  xr  =  offset on theta
%  xc  =  offset on phi
%  dD  =  partial derivatives with respect to phi, theta and scale on
%         (row,col)
%  H   =  Hessian matrix on (row,col)

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }


    % Derivatives computation
    [Iy, Ix] = size(Img1);
    dD = derivSphereScale(Img0, Img1, Img2, row, col, 2*pi/Ix, pi/Iy, 1);
    H = hessianSphere(Img0, Img1, Img2, row, col, 2*pi/Ix, pi/Iy, 1);
    H_inv = inv(H);

    % Extreme calculation
    X = -H_inv * dD;

    xc = X(1,1);
    xr = X(2,1);
    xi = X(3,1);

end
