function [rotatedImg] = rotateSphericalImage(origImg, angle)
% [rotatedImg] = rotateSphericalImage(origImg, angle)
%
% DESCRIPTION:
%  Rotates clockwise a spherical image with respect to the X axis
%
% VERSION:
%  1.1
%
% INPUT:
%  origImg  =  original spherical image
%  angle    =  angle of rotation
%
% OUTPUT:
%  rotatedImg  =  rotated spherical image

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    tic
    disp('[SIFTS] Rotating spherical image...');

    [imgR,imgC,n] = size(origImg);
    [phi, theta] = sphgrid(imgR, imgC);
    
    [newTheta, newPhi] = computeRotatedCoords( theta(:,1)', phi(1,:), ...
                                                pi/2.0, pi, angle);
                                           
    Ir = origImg(:,:,1);
    Ig = origImg(:,:,2);
    Ib = origImg(:,:,3);
    phi(:,end+1) = 2*pi*ones(imgR,1);
    theta(:,end+1) = theta(:,end);
    Ir(:,end+1) = Ir(:,1);
    Ig(:,end+1) = Ig(:,1);
    Ib(:,end+1) = Ib(:,1);
    IrRot = interp2(phi,theta,double(Ir),newPhi,newTheta,'*cubic');
    IgRot = interp2(phi,theta,double(Ig),newPhi,newTheta,'*cubic');
    IbRot = interp2(phi,theta,double(Ib),newPhi,newTheta,'*cubic');
    IrRot(isnan(IrRot))=0;
    IgRot(isnan(IgRot))=0;
    IbRot(isnan(IbRot))=0;
    rotatedImg(:,:,1)=uint8(IrRot);
    rotatedImg(:,:,2)=uint8(IgRot);
    rotatedImg(:,:,3)=uint8(IbRot);

    t=toc;
    disp('[SIFTS] Done!');
    disp(sprintf('[SIFTS] Computation time: %f seconds\n',t));
end
