function [numTotal1, numTotal2, numSame, numCorrectMatch] = testRotationMatching(LSD1, LSD2, samePoints, imgR, imgC, angle)
% [numTotal1,numTotal2,numSame,numCorrectMatch] = testRotationMatching(LSD1,LSD2,samePoints,imgR,imgC,angle)
%
% DESCRIPTION:
%  Gives statistics of matching performance between an omnidirectional 
%  image mapped on the sphere and a rotation of it around the X axis. 
%  Usually, the rotated image is also corrupted by additive Gaussian noise.
%  Criterion of correctness: overlap error < 0.5 (Mikolajczyk et al(2005), "A 
%  comparison of affine region detectors", International Journal of Computer
%  Vision, Vol. 65).
%
% VERSION:
%  1.1
%
% INPUT:
%  LSD1        =  Local Spherical Descriptors of the first image (original
%                 image)
%  LSD2        =  Local Spherical Descriptors of the second image (rotated
%                 image)
%  samePoints  =  matched points
%  imgR        =  number of rows of the spherical grid
%  imgC        =  number of columns of the spherical grid
%  angle       =  angle of rotation between the images
%
% OUTPUT:
%  numTotal1        =  number of Local Spherical Descriptors of the
%                      original image
%  numTotal2        =  number of Local Spherical Descriptors of the rotated
%                      image
%  numSame          =  number of matched descriptors
%  numCorrectMatch  =  number of correctly matched descriptors

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    numTotal1 = length(LSD1);
    numTotal2 = length(LSD2);
    numSame = length(samePoints.index1);

    numCorrectMatch=0;
    
    [phiGrid, thetaGrid] = sphgrid(imgR, imgC);

    for i=1:numSame
        coords1 = [ (2*LSD1(samePoints.index1(i)).imgOptPoints(1)-1)*pi/(2*imgR), ...
                    (LSD1(samePoints.index1(i)).imgOptPoints(2)-1)*2*pi/imgC];

        coords2 = [ (2*LSD2(samePoints.index2(i)).imgOptPoints(1)-1)*pi/(2*imgR), ...
                    (LSD2(samePoints.index2(i)).imgOptPoints(2)-1)*2*pi/imgC];

        [coords2Theta, coords2Phi] = computeRotatedCoords(coords2(:,1)', coords2(:,2)', pi/2.0, pi, angle); 
        clear coords2;
        coords2 = [diag(coords2Theta),diag(coords2Phi)];
        clear coords2Theta;
        clear coords2Phi;

	if(sphericalDistance(coords1(2), coords2(2), coords1(1), coords2(1)) < 1.5*(LSD1(samePoints.index1(i)).sigma+LSD2(samePoints.index2(i)).sigma) )
	        dist1 = sphericalDistance(coords1(2), phiGrid(:), coords1(1), thetaGrid(:));
	        dist2 = sphericalDistance(coords2(2), phiGrid(:), coords2(1), thetaGrid(:));

        	if(computeOverlapError(dist1, dist2, LSD1(samePoints.index1(i)).sigma, LSD2(samePoints.index2(i)).sigma) < 0.5)
        	    numCorrectMatch = numCorrectMatch+1;
		end
        end
    end

    disp('[SIFTS] Testing matching...');
    msg = sprintf('[SIFTS]\t %d points extracted from image 1', numTotal1);
    disp(msg);
    msg = sprintf('[SIFTS]\t %d points extracted from image 2', numTotal2);
    disp(msg);
    msg = sprintf('[SIFTS]\t %d matched points', numSame);
    disp(msg);
    msg = sprintf('[SIFTS]\t %d correctly matched points', numCorrectMatch);
    disp(msg);
    msg = sprintf('[SIFTS]\t %.2f%% of points correctly matched', 100.0*numCorrectMatch/numSame);
    disp(msg);
    msg = sprintf('[SIFTS] Done!\n');
    disp(msg);

end
